﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 7.7.2016
 * Time: 11.02
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;
using TradeServer.ScriptingDriver.ScriptApi.Interfaces;

namespace PshaisSafeties
{
    public class pshaisChandelierExit : ISafetyScript
    {
        int _atrPeriod = 22, _hlPeriod = 22, _interval = 10;
        decimal _atrMult = 3, _exitPosition, _atrValue, _hlValue;
        bool _isLong = true;

    	
        // Properties
        public string Name
        {
            get { return "pshai's Chandelier Exit"; }
        }
        public int TicksBack
        {
        	get { return (_atrPeriod + 21) * _interval; }
        }

        // Setting fields
        public List<ScriptParameter> GetParameters()
        {
            List<ScriptParameter> res = new List<ScriptParameter>
            {
                new ScriptParameter("High/Low Period", ScriptParameterType.Integer, _hlPeriod.ToString()),
                new ScriptParameter("ATR Period", ScriptParameterType.Integer, _atrPeriod.ToString()),
                new ScriptParameter("ATR Multiplier", ScriptParameterType.Decimal, _atrMult.ToString()),
                new ScriptParameter("Interval (minutes)", ScriptParameterType.Integer, _interval.ToString()),
                new ScriptParameter("Is Long Position? (y/n)", ScriptParameterType.String, _isLong ? "y" : "n"),
            };

            return res;
        }

        public void SetParameters(Dictionary<string, object> parameters)
        {
            _hlPeriod = ParseInt(parameters["High/Low Period"], _hlPeriod);
            _atrPeriod = ParseInt(parameters["ATR Period"], _atrPeriod);
            _atrMult = ParseDecimal(parameters["ATR Multiplier"], _atrMult);
            _interval = ParseInt(parameters["Interval (minutes)"], _interval);
            _isLong = ParseString(parameters["Is Long Position? (y/n)"], _isLong ? "y" : "n") == "y";
        }

        // Chart lines
        public List<ChartRecord> GetChartLines()
        {
            if (_isLong)
            {
                return new List<ChartRecord>
                {
                    new ChartRecord(){ChartIndex = 0, HexLineColor = "#FF0000", Name = "Highest High"},
                    new ChartRecord(){ChartIndex = 0, HexLineColor = "#00FF00", Name = "Long Exit"},
                    new ChartRecord() { ChartIndex = 1, HexLineColor = "#FF0000", Name = "ATR" }
                };
            }
            else
            {
                return new List<ChartRecord>
                {
                    new ChartRecord() { ChartIndex = 0, HexLineColor = "#00FF00", Name = "Lowest Low" },
                    new ChartRecord() { ChartIndex = 0, HexLineColor = "#FF0000", Name = "Short Exit" },
                    new ChartRecord() { ChartIndex = 1, HexLineColor = "#FF0000", Name = "ATR" }
                };
            }
        }
        public List<decimal> GetChartData()
        {
            List<decimal> chartData = new List<decimal>
            {
                _hlValue,
                _exitPosition,
                _atrValue
            };

            return chartData;
        }

        // This is called as the script is loaded the first time, aka the init
        public void Init()
        {
        }

        // This is the brain so the say, this is where the calculation is done and an output must be produced.
        public SafetyResult GetResult(SafetyContext context)
        {
            var marketName = string.Format("{0} {1}-{2}", context.PriceSourceName, context.PrimairyCoin,
                context.SecondairyCoin);
            try
            {
                

                // OHLC data
                var open_1min = context.PriceInstrument.Open;
                var high_1min = context.PriceInstrument.High;
                var low_1min = context.PriceInstrument.Low;
                var close_1min = context.PriceInstrument.Close;

                var count = 0;
                var instrument2 = instr(context, _interval, out count, true); // Custom function for higher interval candles in safeties

                // Higher interval candle prices
                var open = instrument2[2];
                var high = instrument2[0];
                var low = instrument2[1];
                var close = instrument2[3];

                // Position in higher interval (in backtests)
                var position = close.Length - (1 + context.Offset / _interval);
                
                // Current close price (in 1min ticks, not higher interval)
                var curClose = Convert.ToDecimal(close_1min[close_1min.Length - (1 + context.Offset)]);

                // Get highest highs and lowest lows
                int hlLB = Core.MaxLookback(_hlPeriod) + 20;
                int hlB, hlL; // begin index, output length
                double[] highs = new double[hlLB + 1];
                double[] lows = new double[hlLB + 1];
                var h_retCode = Core.Max(position - hlLB, position, high, _hlPeriod, out hlB, out hlL, highs);
                var l_retCode = Core.Min(position - hlLB, position, low, _hlPeriod, out hlB, out hlL, lows);

                // Get HL value
                _hlValue = 0.0M;
                if (h_retCode == Core.RetCode.Success && l_retCode == Core.RetCode.Success)
                {
                    if (_isLong)
                        _hlValue = Convert.ToDecimal(highs[hlL - 1]);
                    else
                        _hlValue = Convert.ToDecimal(lows[hlL - 1]);
                }

                // Calculate ATR
                int lb = Core.AtrLookback(_atrPeriod) + 20;
                int b, l; // begin index, output length
                double[] atrOutput = new double[lb + 1];
                var retCode = Core.Atr(position - lb, position, high, low, close, _atrPeriod, out b, out l, atrOutput);

                // Get our ATR and multiply it
                _atrValue = 0.0M;
                if (retCode == Core.RetCode.Success)
                {
                    _atrValue = Convert.ToDecimal(atrOutput[l - 1]) * _atrMult;
                }

                // If ATR and HL value is not >0 then something must have gone wrong, so we would skip this part.
                if (_atrValue > 0 && _hlValue > 0)
                {
                    // If we long...
                    if (_isLong)
                    {
                        // ...get exit position for LONG...
                        _exitPosition = _hlValue - _atrValue;

                        // ...and check for exits.
                        if (curClose < _exitPosition)
                            return SafetyResult.SellShort;
                    }
                    // If we short...
                    else
                    {
                        // ...get exit position for SHORT...
                        _exitPosition = _hlValue + _atrValue;

                        // ...and check for exits.
                        if (curClose > _exitPosition)
                            return SafetyResult.BuyLong;
                    }
                }


            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("{1}:: Exception detected: {0}", ex.Message, marketName));
            }

            return SafetyResult.DoNothing; // Default output
        }

        
        
	    int ParseInt(object value, int lastValue)
	    {
	    	try {
	    		return Convert.ToInt32(value.ToString());
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    double ParseDouble(object value, double lastValue)
	    {
	    	var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			try {
				double output;
				double.TryParse(
					value.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
				return output;
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    decimal ParseDecimal(object value, decimal lastValue)
	    {
	    	var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			try {
				decimal output;
				decimal.TryParse(
					value.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
				return output;
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    string ParseString(object value, string lastValue)
	    {
	    	try {
	    		return Convert.ToString(value);
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }

        //
        // Converts candlestick data to another interval
        List<double[]> instr(SafetyContext context, int mult, out int countOut, bool fullCandlesOnly = false)
        {
            // Get raw input data
            DateTime[] timestamp = context.PriceInstrument.Timestamps;
            double[] high = context.PriceInstrument.High;
            double[] low = context.PriceInstrument.Low;
            double[] open = context.PriceInstrument.Open;
            double[] close = context.PriceInstrument.Close;
            double[] volume = context.PriceInstrument.Volume;
            int interval = 1;
            int compInterval = interval * mult;

            // Output data
            var len = timestamp.Length;
            var ret = new List<double[]>();
            var timestampOut = new List<DateTime>();
            var highOut = new List<double>();
            var lowOut = new List<double>();
            var openOut = new List<double>();
            var closeOut = new List<double>();
            var volumeOut = new List<double>();
            countOut = 0;

            var curTime = DateTime.MinValue;
            var ts = TimeSpan.FromMinutes(compInterval);
            DateTime time;

            for (int i = 0; i < len; i++)
            {
                // Round down the time to match new interval
                try
                {
                    time = RoundDown(timestamp[i], ts);
                }
                catch
                {
                    throw new Exception("DateTime RoundDown failed!");
                }


                if (time > curTime) // Create new "candle"
                {
                    try
                    {
                        timestampOut.Add(time);
                        highOut.Add(high[i]);
                        lowOut.Add(low[i]);
                        openOut.Add(open[i]);
                        closeOut.Add(close[i]);
                        volumeOut.Add(volume[i]);
                        countOut = 1;

                        curTime = time;
                    }
                    catch
                    {
                        throw new Exception("Creating candle failed!");
                    }
                }
                else // ...or edit data for current candle
                {
                    try
                    {
                        var pos = timestampOut.Count - 1;

                        highOut[pos] = Math.Max(highOut[pos], high[i]);
                        lowOut[pos] = Math.Min(lowOut[pos], low[i]);
                        closeOut[pos] = close[i];
                        volumeOut[pos] += volume[i];
                        countOut++;
                    }
                    catch
                    {
                        throw new Exception("Editing candle failed!");
                    }
                }
            }

            if (fullCandlesOnly && countOut < compInterval)
            {
                highOut = highOut.Take(highOut.Count - 1).ToList();
                lowOut = lowOut.Take(lowOut.Count - 1).ToList();
                openOut = openOut.Take(openOut.Count - 1).ToList();
                closeOut = closeOut.Take(closeOut.Count - 1).ToList();
                volumeOut = volumeOut.Take(volumeOut.Count - 1).ToList();
            }

            // Save data and gtfo
            ret.Add(highOut.ToArray());
            ret.Add(lowOut.ToArray());
            ret.Add(openOut.ToArray());
            ret.Add(closeOut.ToArray());
            ret.Add(volumeOut.ToArray());

            return ret;
        }

        // Rounding functions for DateTime
        public static DateTime RoundUp(DateTime dt, TimeSpan ts)
        {
            return Round(dt, ts, true);
        }

        public static DateTime RoundDown(DateTime dt, TimeSpan ts)
        {
            return Round(dt, ts, false);
        }

        private static DateTime Round(DateTime dt, TimeSpan ts, bool up)
        {
            var remainder = dt.Ticks % ts.Ticks;
            if (remainder == 0)
            {
                return dt;
            }

            long delta;
            if (up)
            {
                delta = ts.Ticks - remainder;
            }
            else
            {
                delta = -remainder;
            }

            return dt.AddTicks(delta);
        }
    }
}
